#!/usr/bin/env python
#############################################
# update_site.py - wxWidgets web site updater
# Author: Kevin Ollivier
# License: wxWidgets License
#############################################

""" This module allows us to use non-destructive templates. That means that you do not need to re-generate pages anytime a page changes - only when you change a template. Whenever you alter a template, run this script and it will apply the updates to the whole site.
"""

import sys, os, string, re
import PyMeld
import copy
from section_links import *

root_dir = os.path.abspath(os.path.join("..", "content"))

# First, read in the site template(s)

print "Loading templates..."

template_file = os.path.join(root_dir, "Templates", "template.htm")
template_meld = PyMeld.Meld(open(template_file).read())

section_file = os.path.join(root_dir, "Templates", "section.htm")
section_meld = PyMeld.Meld(open(section_file).read())

def updateDir(dirname):
    """
    This function recursively walks through the subdirectories and
    updates any HTML files it finds.
    """
    for item in os.listdir(dirname):
        fullpath = os.path.join(dirname, item)
        
        if os.path.isfile(fullpath):
            extension = os.path.splitext(fullpath)[1]
            if string.find(extension, "htm") != -1 or string.find(extension, "php") != -1:
                updateFile(fullpath)
        
        elif os.path.isdir(fullpath):
            if item != "Templates" and item != "manuals" and item != "fosdem2003":
                updateDir(fullpath)

def updateFile(filename):
    """
    This imports the existing page contents (minus the template areas) and
    then applies the latest template to those contents.
    """
    global root_dir, template_meld, section_meld
    printable_name = filename[len(root_dir):]
    print "Updating %s" % (printable_name)

    try:

        # Read in the page data we want to keep
        html_meld = PyMeld.Meld(open(filename).read())
        template_copy = template_meld.clone()

        try:
            template_copy.title    = html_meld.title._content
            template_copy.contents = html_meld.contents._content
            template_copy.footer   = html_meld.footer._content
        except ValueError, e:
            print "ValueError: %s" % e
            print "File does not appear to use template: " + printable_name
            return
        except AttributeError, e:
            print "AttributeError: %s" % e
            print "File does not appear to use template: " + printable_name
            return
        except:
            import traceback
            sys.stderr.write(`traceback.print_exc()`)
            print "File does not appear to use template: " + printable_name
            return

        # Check if it's a sub-section of the site, and if so, add the links
        relative_path = string.replace(os.path.dirname(filename), root_dir, "")[1:]

        if relative_path != "":
            names = [folder for folder in relative_path.split(os.sep)]
            # Change this to names[-1] if we want menus for folders deeper than the first level
            section = names[0]
            section_copy = section_meld.clone()
            section_copy.section_name = section.capitalize()
            section_copy.section_links = makeLinksHTML(links[section])
            template_copy.section_box = section_copy

        #if relative_path != "":
        #    template_copy = page_template.replace('href="images', 'href="../images')
        #    template_copy = page_template.replace('src="images', 'src="../images')

        output = open(filename, 'w')
        output.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" ' +
					 '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n')
        output.write(str(template_copy).strip())

    except:

        # just move onto the next file
        sys.stderr.write('WARNING: Unable to update %s\n' % (printable_name))
        import traceback
        sys.stderr.write(`traceback.print_exc()` + '\n')

print "Updating site..."
# Now, update all the pages in the site with this template
updateDir(root_dir)

